/*
 * SPDX-FileCopyrightText: 2006-2021 Istituto Italiano di Tecnologia (IIT)
 * SPDX-FileCopyrightText: 2006-2010 RobotCub Consortium
 * SPDX-License-Identifier: BSD-3-Clause
 */

#ifndef YARP_CONFIG_VERSION_H
#define YARP_CONFIG_VERSION_H

#define YARP_VERSION_MAJOR 3
#define YARP_VERSION_MINOR 7
#define YARP_VERSION_PATCH 0
#define YARP_VERSION_TWEAK 
#define YARP_VERSION "3.7.0~dev"
#define YARP_VERSION_SHORT "3.7.0"

#define YARP_VERSION_ENCODE(MAJOR, MINOR, PATCH) \
    (((MAJOR) * 0x10000) + ((MINOR) * 0x100) + (PATCH))

#define YARP_VERSION_HEX \
    YARP_VERSION_ENCODE(YARP_VERSION_MAJOR, YARP_VERSION_MINOR, YARP_VERSION_PATCH)

#define YARP_VERSION_COMPARE(OP, MAJOR, MINOR, PATCH) \
    (YARP_VERSION_HEX OP YARP_VERSION_ENCODE((MAJOR), (MINOR), (PATCH)))

#endif
