/*
 * SPDX-FileCopyrightText: 2006-2021 Istituto Italiano di Tecnologia (IIT)
 * SPDX-License-Identifier: BSD-3-Clause
 */

#ifndef YARP_OS_IDL_WIRETYPES_H
#define YARP_OS_IDL_WIRETYPES_H

#include <yarp/os/Log.h>
#include <yarp/os/NetFloat32.h>
#include <yarp/os/NetFloat64.h>
#include <yarp/os/NetInt16.h>
#include <yarp/os/NetInt32.h>
#include <yarp/os/NetInt64.h>
#include <yarp/os/NetUint16.h>
#include <yarp/os/NetUint32.h>
#include <yarp/os/NetUint64.h>
#include <yarp/os/Wire.h>
#include <yarp/os/idl/BareStyle.h>
#include <yarp/os/idl/BottleStyle.h>
#include <yarp/os/idl/Unwrapped.h>
#include <yarp/os/idl/WirePortable.h>
#include <yarp/os/idl/WireReader.h>
#include <yarp/os/idl/WireState.h>
#include <yarp/os/idl/WireWriter.h>

#include <cstdint>
#include <cstdio>
#include <map>
#include <set>
#include <string>
#include <vector>

#endif // YARP_OS_IDL_WIRETYPES_H
