/*
 * SPDX-FileCopyrightText: 2006-2021 Istituto Italiano di Tecnologia (IIT)
 * SPDX-License-Identifier: BSD-3-Clause
 */

#ifndef YARP_DEV_PRECISELYTIMED_H
#define YARP_DEV_PRECISELYTIMED_H

#include <yarp/conf/system.h>

YARP_COMPILER_WARNING("<yarp/dev/PreciselyTimed.h> file is deprecated. Use <yarp/dev/IPreciselyTimed.h> instead")
#include <yarp/dev/IPreciselyTimed.h>

#endif // YARP_DEV_PRECISELYTIMED_H
