/*
 * SPDX-FileCopyrightText: 2006-2021 Istituto Italiano di Tecnologia (IIT)
 * SPDX-License-Identifier: BSD-3-Clause
 */

#ifndef YARP_OS_NETUINT8_H
#define YARP_OS_NETUINT8_H

namespace yarp::os {

/**
 * Definition of the NetUint8 type
 */
typedef std::uint8_t NetUint8;

} // namespace yarp::os

#endif // YARP_OS_NETUINT8_H
