/*
 * SPDX-FileCopyrightText: 2006-2021 Istituto Italiano di Tecnologia (IIT)
 * SPDX-FileCopyrightText: 2006-2010 RobotCub Consortium
 * SPDX-License-Identifier: BSD-3-Clause
 */

#ifndef YARP_DEV_AUDIOGRABBERINTERFACES_H
#define YARP_DEV_AUDIOGRABBERINTERFACES_H

#include <yarp/dev/AudioGrabberVocabs.h>
#include <yarp/dev/IAudioGrabberSound.h>
#include <yarp/dev/IAudioRender.h>

#endif // YARP_DEV_AUDIOGRABBERINTERFACES_H
