/*
 * SPDX-FileCopyrightText: 2006-2021 Istituto Italiano di Tecnologia (IIT)
 * SPDX-License-Identifier: BSD-3-Clause
 */

YARP_COMPILER_DEPRECATED_WARNING(<yarp/conf/option.h> header is deprecated.)
