/*
 * SPDX-FileCopyrightText: 2006-2021 Istituto Italiano di Tecnologia (IIT)
 * SPDX-License-Identifier: BSD-3-Clause
 */

#ifndef YARP_MANAGER_SINGLEAPPLOADER
#define YARP_MANAGER_SINGLEAPPLOADER

#include <yarp/manager/ymm-types.h>
#include <yarp/manager/manifestloader.h>

namespace yarp::manager {


/**
 * Class SingleAppLoader
 */
class SingleAppLoader : public AppLoader {

public:
    SingleAppLoader(const char* szModule, const char* strHost);
    ~SingleAppLoader() override;
    bool init() override;
    void fini() override;
    Application* getNextApplication() override;

protected:

private:
    Application app;
    std::string strModule;
    std::string strHost;
};

} // namespace yarp::manager


#endif // __YARP_MANAGER_SINGLEAPPLOADER__
