if (Test-Path env:COMSPEC) {
    $Env:YARP_DATA_DIRS+=";$(Join-Path -Path $Env:CONDA_PREFIX -ChildPath Library\share\yarp)"
    $Env:XDG_DATA_DIRS+=";$(Join-Path -Path $Env:CONDA_PREFIX -ChildPath Library\share)"
}
else {
    $Env:YARP_DATA_DIRS+=";$(Join-Path -Path $Env:CONDA_PREFIX -ChildPath share/yarp)"
    $Env:XDG_DATA_DIRS+=";$(Join-Path -Path $Env:CONDA_PREFIX -ChildPath share)"
}

