/*
 * SPDX-FileCopyrightText: 2006-2021 Istituto Italiano di Tecnologia (IIT)
 * SPDX-License-Identifier: BSD-3-Clause
 */

#ifndef YARP_DEV_CALIBRATORINTERFACES_H
#define YARP_DEV_CALIBRATORINTERFACES_H

#include <yarp/dev/ICalibrator.h>
#include <yarp/dev/IRemoteCalibrator.h>
#include <yarp/dev/CalibratorVocabs.h>

#endif // YARP_DEV_CALIBRATORINTERFACES_H
