if [ ! -z "$ZSH_VERSION" ]; then
  SCRIPT_DIR=${0:a:h}
  SCRIPT_NAME=$(basename "${(%):-%x}")
  SCRIPT_NAME_WITHOUT_EXT=${SCRIPT_NAME%.*}
  source ${SCRIPT_DIR}/${SCRIPT_NAME_WITHOUT_EXT}.zsh
else
  SCRIPT_DIR="$( cd "$( dirname "${BASH_SOURCE[0]}" )" &> /dev/null && pwd )"
  SCRIPT_NAME=$(basename "${BASH_SOURCE[0]}")
  SCRIPT_NAME_WITHOUT_EXT=${SCRIPT_NAME%.*}
  source ${SCRIPT_DIR}/${SCRIPT_NAME_WITHOUT_EXT}.bash
fi
